# frozen_string_literal: true

require 'test_helper'

class WorkoutTest < ActiveSupport::TestCase
  def setup
    @workout = FactoryBot.build(:workout)
  end

  def test_factory_build_can_be_saved
    assert @workout.save, "errors: #{@workout.errors.messages}"
  end

  ### ASSOCIATIONS
  def test_has_many_elements
    assert_respond_to @workout, :elements
  end

  def test_has_many_movements
    assert_respond_to @workout, :movements
  end

  def test_can_add_workout_elements
    assert_empty @workout.elements
    assert_empty @workout.movements

    @workout.elements << FactoryBot.build(:workout_element)

    assert @workout.save

    assert_not_empty @workout.elements
    assert_not_empty @workout.movements
  end

  def test_destroy_destroys_workout_elements
    workout = FactoryBot.create(:workout)

    FactoryBot.create(:workout_element, workout: workout)

    assert_changes 'WorkoutElement.count', from: 1, to: 0 do
      workout.destroy
    end
  end

  def test_destroy_does_not_destroy_movements
    movement = FactoryBot.create(:movement)
    workout = FactoryBot.create(:workout)

    FactoryBot.create(:workout_element, workout: workout, movement: movement)

    assert_no_changes 'Movement.count' do
      workout.destroy
    end
  end

  ### ATTRIBUTES
  def test_aliases_rounds_to_sets
    @workout.sets = 5

    assert_equal 5, @workout.rounds
  end

  ### VALIDATIONS
  def test_measure_cannot_be_arbitrary
    @workout.measure = :time

    assert @workout.valid?

    @workout.measure = :happyness

    refute @workout.valid?
  end

  test 'interval should be positive' do
    [-1, 0].each do |n|
      @workout.interval = n
      refute @workout.valid?, "interval should be greater than zero, which '#{n}' isn't"
    end

    @workout.interval = 1
    assert @workout.valid?
  end

  ### INSTANCE METHODS
  test 'should write and read repetition scheme' do
    @workout.repetition_scheme = ''

    assert_equal [], @workout.repetition_scheme, 'repetition scheme cannot be empty'

    [
      [21, 15, 9],
      '21-15-9]',
      '21 15 9'
    ].each do |input|
      @workout.repetition_scheme = input
    end

    assert_equal [21, 15, 9], @workout.repetition_scheme
  end

  test 'should correctly identify as EMOM' do
    @workout.frequency = :start_every
    @workout.interval = 60

    assert @workout.emom?
  end

  test 'should correctly identify as AMRAP' do
    @workout.frequency = :continuous
    @workout.time = 600

    assert @workout.amrap?
  end
end
